<?php
/**
 *  A helper class dedicated to the JSON Web Token (JWT) which can load
 *  the token from a file or request a new JWT from the Copernica 
 *  authenticate server. Note that it's main purpose after loading the
 *  token is to check if it has not expired (not verifying signature).
 * 
 *  @copyright    Copernica BV 2023
 */

/**
 *  Class implementation
 */
class JSONWebToken
{
    /**
     *  The payload of the JWT
     *  @var   array
     */
    private $payload;

    /**
     *  The raw JWT data
     *  @var    string
     */
    private $raw;


    /**
     *  Class constructor.
     * 
     *  @param  string $data    the JWT token as a string
     *  @throws on invalid token
     */
    public function __construct(string $data)
    {
        // attempt to split into the three parts
        if (count($parts = explode(".", trim($data))) != 3) throw new Exception('The JWT has an incorrect structure!');

        // get the header and payload
        list($header, $payload) = $parts;
        
        // get the different sections
        $this->payload = json_decode(self::decode($payload), true);
        
        // the payload and headers must be strings
        if (!is_array($this->payload)) throw new Exception('Invalid JSON in payload');

        // remember the raw JWT as well
        $this->raw = $data;
    }

    /**
     *  Alternative constructor that does not throw.
     * 
     *  @return self|null
     */
    public static function parse(string $data): ?JSONWebToken
    {
        // prevent exceptions
        try
        {
            // create the token
            return new self($data);
        }

        // handle exceptions
        catch (Exception $exception)
        {
            // token cannot be constructed
            return null;
        }
    }

    /**
     *  Helper method that attempts to get a JWT from the Copernica 
     *  authenticate server. This is achieved by sending a POST request
     *  containing the access token in the body of the request.
     *  @param  string    the API access token 
     *  @return JSONWebToken|null
     */
    public static function fromServer(string $access_token): ?JSONWebToken
    {
        // initialize the curl resource
        $curl = curl_init("https://authenticate.copernica.com");

        // options for the curl resource
        $options = array(
            CURLOPT_POST            =>  true,
            CURLOPT_RETURNTRANSFER  =>  true,
            CURLOPT_POSTFIELDS      =>  ['access_token' => $access_token]
        );

        // set the defined options
        curl_setopt_array($curl, $options);

        // execute the curl request
        $response = curl_exec($curl);

        // retrive the resulting HTTP response code
        $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);

        // clean up the curl resource
        curl_close($curl);

        // bad request
        if ($httpCode !== 200) return null;

        // done
        return self::parse($response);
    }

    /**
     *  Method to determine whether this token has expired.
     *  @return bool
     */
    public function expired(): bool
    {
        // check expire time against current time
        return time() > $this->payload['exp'];
    }

    /**
     *  Expose the raw JWT data.
     *  @return string
     */
    public function raw(): string
    {
        return $this->raw;
    }

    /**
     *  Helper to provide base64 URL decoding support which PHP does not
     *  have natively.
     *  @param  string $data    url encoded input data
     *  @return string          decoded data 
     *  @throws Exception
     */
    private static function decode(string $data): string
    {
        // decode the Base64Url string
        $result = base64_decode(str_pad(strtr($data, '-_', '+/'), strlen($data) % 4, '=', STR_PAD_RIGHT));
        
        // check for error
        if ($result === false) throw new Exception("failed to decode JWT");
        
        // expose the result
        return $result;
    }
}
